# &#9888; This package is work in progress. Please do not use it yet &#9888;<br>

# Voca design system foundations

This package contains the most atomic building blocks of Voca design system - tokens & variables. Design tokens are
platform-agnostic entities that are defined and used by the designers. They also act as a source for
the [style-dictionary](https://amzn.github.io/style-dictionary/#/) to generate usable files and variables for the web
platform and possibly more.<br>

Tokens and the generated variables are classified by
categories: `border, breakpoint, color, motion, shadow, spacing, typography`. <br>

## Tokens

Tokens are located in `tokens` directory in the form of JSON files. There are two types of tokens in our system -
base & semantic. `tokens/base` represent our core set of values & scales, while `tokens/semantic` only reference the
base tokens, however they carry design intent and context.<br>
Tokens are structured following the CTI pattern - Category, Type, Item. Meaning that top level properties in JSON file
are Category objects, their properties are Type objects and the Type properties are Item objects, containing the actual
token value. <br>

```json
{
  "category": {
    "type": {
      "item": {
        "value": "token_value"
      }
    }
  }
}
```

In certain cases, like semantic color tokens, there can be an additional Sub-item level.

## Variables

Each category as a separate directory has a set of consumable files containing variables:

1. `.css` - global CSS variables
2. `.scss` - SCSS variables
3. `.js` - ES6 named exports for each variable
4. `.d.ts` - TS variable type declarations
5. `.json` - variables in JSON format (note: it's different from token JSON files in `tokens/`)
6. ... If you need additional format, please, feel free to contact our team with a request

## Usage

### CSS

`.css` files contain CSS variables (custom properties) which are applied at the `:root` of the document upon import or
load of our stylesheet. Meaning they will be globally available due to CSS cascade & inheritance feature, so you don't
have to re-import them in every file that references these variables.

```css
/* import all variables anywhere in the root of your app once*/
@import '~@teliads/foundations/index.css'; /* Webpack */
/* or only specific category */
@import '~@teliads/foundations/color/variables.css'; /* Webpack */
/* TBA for other bundlers */

/* then refer anywhere in your app */
.button {
  background-color: var(--vds-color-background-interactive-primary);

  /* or with a fallback, if required */
  background-color: var(--vds-color-background-interactive-primary, purple);
}
```

Note: loading all the variables could make browser dev-tools experience unpleasant for some, since all the variables will
be visible when inspecting styles.

### SCSS

`.scss` files contain SCSS variables, which are different from CSS variables and should be consumed in a different
manner, as they are scoped to the module they're declared in. So in order to use our SCSS variables you need to load
them in every module you reference them.

```scss
// import all the variables to your scss module (file)
@use '~@teliads/foundations'; // Webpack

// use variables through imported namespace
.button {
  background-color: foundations.$vds-color-background-interactive-primary;
}
```

```scss
// or import only a single category
@use '~@teliads/foundations/color/variables';

.button {
  background-color: variables.$vds-color-background-interactive-primary;
}
```

```scss
// or with a custom namespace
@use '~@teliads/foundations/color/variables' as colors; // Webpack

.button {
  background-color: colors.$vds-color-background-interactive-primary;
}
```

[\* More on SCSS Namespaces](https://sass-lang.com/documentation/at-rules/use#choosing-a-namespace) <br>
&#9888; You might be tempted to use `@import` rule to define the variables globally, however refrain from doing so, as this rule
[will be discontinued](https://github.com/sass/sass/blob/main/accepted/module-system.md#timeline).

### Javascript / Typescript

Javascript files contain variables which are exported individually as named exports. There's a type declaration file
for Typescript to resolve automatically if you're using typescript. But the usage remains the same for JS and TS users.

```typescript
// React + styled-components

// you can import individual variables from barrel file
import { vdsColorBackgroundInteractivePrimary } from '@teliads/foundations';

const Button = styled.button`
  background-color: ${vdsColorBackgroundInteractivePrimary};
`;
```

```typescript
// or namespace all of the variables and refer to them via namespace object
import * as vdsVariables from '@teliads/foundations';

const Button = styled.button`
  background-color: ${vdsVariables.vdsColorBackgroundInteractivePrimary};
`;
```

```typescript
// or you can import only the category you need
import { vdsColorBackgroundInteractivePrimary } from '@teliads/foundations/color/variables';

const Button = styled.button`
  background-color: ${vdsColorBackgroundInteractivePrimary};
`;
```

### JSON

In case you need to use variables in a more language agnostic context, you can use `.json` files containing flattened
variable inventory or use the design tokens themselves, which are located in the `tokens` directory. The actual usage
is highly dependent on your use case and the context you're using the file in.
